"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const opn = require("opn");
const express = require("express");
const fs_1 = require("fs");
const path_1 = require("path");
exports.server = {
    active: false,
    loading: false,
    instance: null
};
const app = express();
const port = 8080;
let tempContainer = [];
const plotContainer = {};
function clear() {
    tempContainer = [];
}
exports.clear = clear;
function stack(data, layout) {
    const container = layout ? { data, layout } : { data };
    tempContainer.push(container);
}
exports.stack = stack;
function plot(data, layout, cb) {
    if (data) {
        const container = layout ? { data, layout } : { data };
        tempContainer.push(container);
    }
    const id = Object.keys(plotContainer).length;
    plotContainer[id] = {
        opened: false,
        pending: false,
        plots: tempContainer
    };
    tempContainer = [];
    spawn(() => {
        if (cb) {
            cb(id);
        }
    });
}
exports.plot = plot;
function spawn(cb) {
    if (!exports.server.active && !exports.server.loading) {
        exports.server.loading = true;
        app.get('/data/:id', (req, res) => {
            const requestId = req.params.id;
            const container = plotContainer[requestId];
            const plots = container && container.plots;
            plotContainer[requestId].opened = true;
            plotContainer[requestId].pending = false;
            res.send(plots);
            close();
        });
        app.get('/plots/:id', (req, res) => {
            const plotId = req.params.id;
            fs_1.readFile(path_1.join(__dirname, 'www', 'index.html'), 'utf-8', (error, file) => {
                fs_1.readFile(path_1.join(__dirname, 'www', 'script.js'), 'utf-8', (err, script) => {
                    script = script.replace('{{plotId}}', `${plotId}`);
                    file = file.replace('{{script}}', script);
                    return res.send(file);
                });
            });
        });
        exports.server.instance = app.listen(port, () => {
            exports.server.active = true;
            exports.server.loading = false;
            openPlots();
            cb();
        });
    }
    else if (exports.server.active) {
        openPlots();
        cb();
    }
}
function openPlots() {
    for (const plotEntry of Object.entries(plotContainer)) {
        if (!plotEntry[1].opened && !plotEntry[1].pending) {
            plotEntry[1].pending = true;
            opn(`http://localhost:${port}/plots/${plotEntry[0]}`);
        }
    }
}
function close() {
    const pending = Object.values(plotContainer)
        .map(o => o.pending)
        .reduce((a, b) => a || b);
    const opened = Object.values(plotContainer)
        .map(o => o.opened)
        .reduce((a, b) => a && b);
    if (exports.server.instance && exports.server.active && !pending && opened) {
        exports.server.instance.close(() => {
            exports.server.active = false;
        });
    }
}
//# sourceMappingURL=index.js.map